/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "graphics.image.h"
#include "graphics3d.context.h"
#include "graphics3d.texture.h"
#include "graphics3d.vertexformat.h"

#include "object.inl"
#include "base.string.inl"
#include "system.thread.inl"
#include "system.assembler.inl"
#include "graphics3d.context.inl"
#include "graphics3d.texture.inl"
#include "graphics3d.resource.inl"
#include "graphics3d.vertexformat.inl"


namespace magnum { namespace graphics3d {
using magnum::graphics::Image;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Texture::Texture(unsigned int tex, unsigned int target) {
	int tmp;

	m_OglTexture = tex;
	m_OglTarget = target;
	m_MagFilter = Nearest;
	m_MinFilter = Nearest;
	m_WrapS = Repeat;
	m_WrapT = Repeat;
	m_WrapU = Repeat;
	m_MinLod = -1000.0f;
	m_MaxLod = 1000.0f;
	m_LodBias = 0.0f;

	glActiveTexture(GL_TEXTURE0);
	glBindTexture(m_OglTarget, tex);
	glGetTexLevelParameteriv(m_OglTarget, 0, GL_TEXTURE_INTERNAL_FORMAT, &tmp);
	switch (tmp) {
		case 1: m_Type = Image::Index8; break;
		case GL_RGB8: m_Type = Image::RGB8; break;
		case GL_RGBA8: m_Type = Image::RGBA8; break;
		default: m_Type = Image::Unknown;
	}
	glGetTexParameteriv(m_OglTarget, GL_TEXTURE_MAX_LEVEL, &tmp);
	m_Levels = tmp+1;
	
	glTexParameteri(m_OglTarget, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(m_OglTarget, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(m_OglTarget, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(m_OglTarget, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(m_OglTarget, GL_TEXTURE_WRAP_R, GL_REPEAT);
	glTexParameterf(m_OglTarget, GL_TEXTURE_MIN_LOD, m_MinLod);
	glTexParameterf(m_OglTarget, GL_TEXTURE_MAX_LOD, m_MaxLod);
	
	getContext()->getTextureStack()->m_Texture[0] = this;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Texture::~Texture() {
	// Remove texture from all contexts!
	TextureStack* ts = getContext()->getTextureStack();
	for (int i = 0; i < MaxTextures; i++) {
		if (ts->m_Texture[i] == this)
			ts->m_Texture[i] = NULL;
	}
	glDeleteTextures(1, &m_OglTexture);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Texture::compileSelect(Assembler& as, unsigned int stage) const {
	as.asmCall((void (Texture::*)(unsigned int ) const)Texture::forceSelect, this, stage);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Texture::setFilter(Filter min, Filter mag) {
	m_MinFilter = min;
	m_MagFilter = mag;
	int oldtex;
	glGetIntegerv(m_OglTarget, &oldtex);
	glBindTexture(m_OglTarget, m_OglTexture);
	glTexParameteri(m_OglTarget, GL_TEXTURE_MIN_FILTER, min);
	glTexParameteri(m_OglTarget, GL_TEXTURE_MAG_FILTER, mag);
	glBindTexture(m_OglTarget, oldtex);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Texture::setWrapping(Wrapping s, Wrapping t, Wrapping u) {
	m_WrapS = s;
	m_WrapT = t;
	m_WrapU = u;
	int oldtex;
	glGetIntegerv(m_OglTarget, &oldtex);
	glBindTexture(m_OglTarget, m_OglTexture);
	glTexParameteri(m_OglTarget, GL_TEXTURE_WRAP_S, s);
	glTexParameteri(m_OglTarget, GL_TEXTURE_WRAP_T, t);
	glTexParameteri(m_OglTarget, GL_TEXTURE_WRAP_R, u);
	glBindTexture(m_OglTarget, oldtex);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Texture::setLod(float mi, float ma) {
	m_MinLod = mi;
	m_MaxLod = ma;
	int oldtex;
	glGetIntegerv(m_OglTarget, &oldtex);
	glBindTexture(m_OglTarget, m_OglTexture);
	glTexParameterf(m_OglTarget, GL_TEXTURE_MIN_LOD, mi);
	glTexParameterf(m_OglTarget, GL_TEXTURE_MAX_LOD, ma);
	glBindTexture(m_OglTarget, oldtex);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Texture::setLodBias(float bias) {
	m_LodBias = bias;
	int oldtex;
	glGetIntegerv(m_OglTarget, &oldtex);
	glBindTexture(m_OglTarget, m_OglTexture);
	glTexParameterf(m_OglTarget, GL_TEXTURE_LOD_BIAS, bias);
	glBindTexture(m_OglTarget, oldtex);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
int Texture::mapOutputFormat(Type t) {
	switch (t) {
		case Image::BGRA8:
			return GL_RGBA8;
		case Image::BGR8:
			return GL_RGB8;
		case Image::RGBA8:
			return GL_RGBA8;
		case Image::RGB8:
			return GL_RGB8;
		case Image::R5G6B5:
			return GL_RGB5;
		case Image::RGB5:
			return GL_RGB5;
		case Image::RGB16:
			return GL_RGB16;
		case Image::RGBA16:
			return GL_RGBA16;
		case Image::BGR16:
			return GL_RGB16;
		case Image::BGRA16:
			return GL_RGBA16;
		case Image::Index8:
			return 1;
		case Image::A8:
			return GL_ALPHA8;
		case Image::A16:
			return GL_ALPHA16;
		case Image::DUDV8:
			return GL_DU8DV8_ATI;
		default:
			return -1;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
int Texture::mapInputFormat(Type t) {
	switch (t) {
		case Image::BGRA8:
			return GL_BGRA_EXT;
		case Image::BGR8:
			return GL_BGR_EXT;
		case Image::RGBA8:
			return GL_RGBA;
		case Image::RGB8:
			return GL_RGB;
		case Image::R5G6B5:
			return GL_RGB;
		case Image::RGB5:
			return GL_RGB;
		case Image::RGB16:
			return GL_RGB;
		case Image::RGBA16:
			return GL_RGBA;
		case Image::BGR16:
			return GL_BGR_EXT;
		case Image::BGRA16:
			return GL_BGRA_EXT;
		case Image::Index8:
			return GL_COLOR_INDEX;
		case Image::DUDV8:
			return GL_DUDV_ATI;
		case Image::A8:
			return GL_ALPHA;
		case Image::A16:
			return GL_ALPHA;
		default:
			return -1;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
int Texture::mapInputType(Type t) {
	switch (t) {
		case Image::BGRA8:
			return GL_UNSIGNED_BYTE;
		case Image::BGR8:
			return GL_UNSIGNED_BYTE;
		case Image::RGBA8:
			return GL_UNSIGNED_BYTE;
		case Image::RGB8:
			return GL_UNSIGNED_BYTE;
		case Image::R5G6B5:
			return GL_UNSIGNED_SHORT_5_6_5;
		case Image::RGB5:
			return GL_UNSIGNED_SHORT_5_5_5_1;
		case Image::RGB16:
			return GL_UNSIGNED_SHORT;
		case Image::RGBA16:
			return GL_UNSIGNED_SHORT;
		case Image::BGR16:
			return GL_UNSIGNED_SHORT;
		case Image::BGRA16:
			return GL_UNSIGNED_SHORT;
		case Image::Index8:
			return GL_UNSIGNED_BYTE;
		case Image::DUDV8:
			return GL_BYTE;
		case Image::A8:
			return GL_UNSIGNED_BYTE;
		case Image::A16:
			return GL_UNSIGNED_SHORT;
		default:
			return -1;
	}
}

}; };
